import os
import sys
import json

import pytest
from click.testing import CliRunner

from chalice.cli import newproj


try:
    from aws_cdk import core as cdk
except Exception:
    pytestmark = pytest.mark.skip(
        "aws_cdk package needed to run CDK tests.")


@pytest.fixture
def runner():
    return CliRunner()


def load_chalice_construct(dirname, stack_name='testcdk'):
    try:
        sys.path.append(dirname)
        sys.modules.pop('stacks.chaliceapp', None)
        sys.modules.pop('stacks', None)
        import stacks.chaliceapp
        app = cdk.App()
        chalice_app = stacks.chaliceapp.ChaliceApp(app, stack_name)
        return app, chalice_app.chalice
    finally:
        sys.modules.pop('app', None)
        sys.modules.pop('stacks', None)
        sys.path.pop()
        sys.path_importer_cache.clear()


def filter_resources(template, resource_type):
    return [(name, props) for name, props in template['Resources'].items()
            if props['Type'] == resource_type]


def test_cdk_construct_api(runner):
    # The CDK loading/synth can take a while so we're testing the
    # various APIs out in one test to cut down on test time.
    with runner.isolated_filesystem():
        newproj.create_new_project_skeleton(
            'testcdk', project_type='cdk-ddb')
        dirname = os.path.abspath(os.path.join('testcdk', 'infrastructure'))
        os.chdir(dirname)
        cdk_app, chalice_app = load_chalice_construct(dirname, 'testcdk')
        api_handler = chalice_app.get_function('APIHandler')
        assert api_handler == chalice_app.get_function('APIHandler')
        role = chalice_app.get_role('DefaultRole')
        assert hasattr(role, 'role_name')


def test_can_package_as_cdk_app(runner):
    # The CDK loading/synth can take a while so we're testing the
    # various APIs out in one test to cut down on test time.
    with runner.isolated_filesystem():
        newproj.create_new_project_skeleton(
            'testcdkpackage', project_type='cdk-ddb')
        dirname = os.path.abspath(os.path.join('testcdkpackage',
                                               'infrastructure'))
        os.chdir(dirname)
        cdk_app, chalice_app = load_chalice_construct(
            dirname, 'testcdkpackage')
        assembly = cdk_app.synth()
        stack = assembly.get_stack_by_name('testcdkpackage')
        cfn_template = stack.template
        resources = cfn_template['Resources']
        # Sanity check that we have the resources from Chalice as well as the
        # resources from the ChaliceApp construct.
        assert 'APIHandler' in resources
        assert 'DefaultRole' in resources
        ddb_tables = filter_resources(cfn_template, 'AWS::DynamoDB::Table')[0]
        # CDK adds a random suffix to our resource name so we verify that
        # the name starts with our provided name "AppTable".
        assert ddb_tables[0].startswith('AppTable')
        # We also need to verify that we've replaces the CodUri with the
        # CDK specific assets.
        functions = filter_resources(
            cfn_template, 'AWS::Serverless::Function')[0]
        bucket_ref = functions[1]['Properties']['CodeUri']['Bucket']['Ref']
        assert bucket_ref.startswith('AssetParameters')


def test_can_package_managed_layer(runner):
    with runner.isolated_filesystem():
        newproj.create_new_project_skeleton(
            'testcdklayers', project_type='cdk-ddb')
        project_dir = os.path.abspath('testcdklayers')
        config_file = os.path.join(project_dir, 'runtime',
                                   '.chalice', 'config.json')
        with open(config_file) as f:
            config = json.load(f)
            config['automatic_layer'] = True
        with open(config_file, 'w') as f:
            f.write(json.dumps(config))
        infrastructure_dir = os.path.join(project_dir, 'infrastructure')
        os.chdir(infrastructure_dir)
        cdk_app, chalice_app = load_chalice_construct(infrastructure_dir,
                                                      'testcdklayers')
        assembly = cdk_app.synth()
        stack = assembly.get_stack_by_name('testcdklayers')
        cfn_template = stack.template
        layers = filter_resources(
            cfn_template, 'AWS::Serverless::LayerVersion')[0]
        bucket_ref = layers[1]['Properties']['ContentUri']['Bucket']['Ref']
        assert bucket_ref.startswith('AssetParameters')



import os

import pytest

from chalice.cli import newproj


class InMemoryOSUtils(object):
    def __init__(self, filemap=None):
        if filemap is None:
            filemap = {}
        self.filemap = filemap
        self.walk_return_val = None

    def dirname(self, name):
        return os.path.dirname(name)

    def get_directory_contents(self, dirname):
        full_paths = [f for f in self.filemap if f.startswith(dirname)]
        return [p.split(os.sep)[1] for p in full_paths]

    def file_exists(self, filename):
        return filename in self.filemap

    def joinpath(self, *args):
        return os.path.join(*args)

    def walk(self, root_dir):
        return self.walk_return_value

    def directory_exists(self, dirname):
        return True

    def get_file_contents(self, filename, binary=True):
        return self.filemap[filename]

    def set_file_contents(self, filename, contents, binary=True):
        self.filemap[filename] = contents


@pytest.mark.parametrize(
    'contents,template_kwargs,expected', [
        ('{{myvar}}', {'myvar': 'foo'}, 'foo'),
        ('{{myvar}}', {'myvar': 'foo', 'myvar2': 'bar'}, 'foo'),
        ('before {{myvar}} after', {'myvar': 'foo'}, 'before foo after'),
        ('newlines\n{{myvar}}\nbar', {'myvar': 'foo'}, 'newlines\nfoo\nbar'),
        ('NAME = "{{myvar}}"', {'myvar': 'foo'}, 'NAME = "foo"'),
        ('{{one}}{{two}}', {'one': 'foo', 'two': 'bar'}, 'foobar'),
        ('{nomatch}', {'nomatch': 'bar'}, '{nomatch}'),
        ('no template', {'nomatch': 'bar'}, 'no template'),
        ('', {}, ''),
        ('{{noclose', {}, '{{noclose'),
        ('nostart}}', {}, 'nostart}}'),
        ('{{unknown_var}}', {}, newproj.BadTemplateError()),
    ]
)
def test_can_get_templated_content(contents, template_kwargs, expected):
    if isinstance(expected, Exception):
        with pytest.raises(expected.__class__):
            newproj.get_templated_content(contents, template_kwargs)
    else:
        newproj.get_templated_content(contents, template_kwargs) == expected


def test_newproj_copies_and_templates_files():
    fake_osutils = InMemoryOSUtils()
    fake_osutils.walk_return_value = [
        ('source_dir', [], ['foo', 'bar']),
    ]
    fake_osutils.filemap = {
        os.path.join('source_dir', 'foo'): 'hello',
        os.path.join('source_dir', 'bar'): '{{who}}',
    }
    creator = newproj.ProjectCreator(fake_osutils)
    creator.create_new_project('source_dir', 'dest_dir', {'who': 'world'})
    assert fake_osutils.filemap[os.path.join('dest_dir', 'foo')] == 'hello'
    assert fake_osutils.filemap[os.path.join('dest_dir', 'bar')] == 'world'


def test_can_list_available_projects():
    fake_osutils = InMemoryOSUtils()
    join = os.path.join
    first_dir = join('template-dir', '0001-first-proj')
    second_dir = join('template-dir', '0002-second-proj')
    fake_osutils.filemap = {
        join(first_dir, 'metadata.json'): '{"description": "First template"}',
        join(second_dir, 'metadata.json'): '{"description": "Second"}',

    }
    results = newproj.list_available_projects('template-dir', fake_osutils)
    assert results == [
        newproj.ProjectTemplate(
            dirname='0001-first-proj',
            metadata={'description': 'First template'},
            key='first-proj',
        ),
        newproj.ProjectTemplate(
            dirname='0002-second-proj',
            metadata={'description': 'Second'},
            key='second-proj',
        ),
    ]



import os
import json

import pytest
from click.testing import CliRunner

from chalice.cli import newproj
from chalice.api import package_app


@pytest.fixture
def runner():
    return CliRunner()


@pytest.mark.parametrize('package_format,template_format,expected_filename', [
    ('cloudformation', 'json', 'sam.json'),
    ('cloudformation', 'yaml', 'sam.yaml'),
    ('terraform', 'json', 'chalice.tf.json'),
])
def test_can_package_different_formats(runner, package_format,
                                       template_format,
                                       expected_filename):
    with runner.isolated_filesystem():
        newproj.create_new_project_skeleton('testproject')
        package_app('testproject', output_dir='packagedir', stage='dev',
                    package_format=package_format,
                    template_format=template_format)
        app_contents = os.listdir('packagedir')
        assert expected_filename in app_contents
        assert 'deployment.zip' in app_contents


def test_can_override_chalice_config(runner):
    with runner.isolated_filesystem():
        newproj.create_new_project_skeleton('testproject')
        chalice_config = {
            'environment_variables': {
                'FOO': 'BAR',
            }
        }
        package_app('testproject', output_dir='packagedir', stage='dev',
                    chalice_config=chalice_config)
        app_contents = os.listdir('packagedir')
        assert 'sam.json' in app_contents
        with open(os.path.join('packagedir', 'sam.json')) as f:
            data = json.loads(f.read())
        properties = data['Resources']['APIHandler']['Properties']
        assert properties['Environment'] == {
            'Variables': {
                'FOO': 'BAR',
            }
        }
